package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.constant.CtopRoleCodeConstant;
import cn.com.ctop.common.module.entity.Advertiser;
import cn.com.ctop.common.module.entity.MaterialInfo;
import cn.com.ctop.common.module.entity.Project;
import cn.com.ctop.common.module.entity.SysUser;
import cn.com.ctop.common.module.mapper.ProjectMapper;
import cn.com.ctop.common.module.service.*;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.ctop.entity.ProjectMember;
import org.jeecg.modules.ctop.service.IAdvertiserService;
import org.jeecg.modules.ctop.service.IProjectMemberService;
import org.jeecg.modules.ctop.service.ITransferAccountService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 项目
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-11-19
 */
@Slf4j
@Api(tags = "项目")
@RestController
@RequestMapping("/ctop/project")
public class ProjectController {
    @Autowired
    private IProjectService projectService;
    @Autowired
    private IProjectMemberService projectMemberService;

    @Autowired
    private IAdvertiserService advertiserService;
    @Autowired
    private ProjectMapper projectMapper;
    @Autowired
    private ITransferAccountService transferAccountService;
    @Autowired
    private IUserAllocationService userAllocationService;
    @Autowired
    private ISysUserService userService;

    /**
     * @param advertiserId
     * @param mediaId
     * @return
     */
    @GetMapping(value = "/getProjectByAdvertiserAndMediaId")
    public Result<List<Project>> getProjectByCompany(String advertiserId, String mediaId) {
        Result<List<Project>> result = new Result<>();
        try {
            QueryWrapper<Project> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("media_id", mediaId);
            queryWrapper.eq("advertiser_id", advertiserId);
            List<Project> projects = projectMapper.selectList(queryWrapper);
            result.setSuccess(true);
            result.setResult(projects);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
        }
        return result;
    }


    @Autowired
    private IProjectMemberService memberService;

    @AutoLog(value = "项目-分页列表查询")
    @ApiOperation(value = "项目-分页列表查询", notes = "项目-分页列表查询")
    @GetMapping(value = "/responsibleListV2")
    public Result<IPage<Project>> responsibleListV2(Project project,
                                                    @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                    @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                    HttpServletRequest req) {
        Result<IPage<Project>> result = new Result<>();
        String userId = project.getUserId();
        if (Check.isNull(userId)) {
            result.setSuccess(false);
            result.setMessage("用户信息不能为空");
            return result;
        }
        project.setUserId(null);
        String projectName = project.getProjectName();
        project.setProjectName(null);
        String roleCode = sysRoleService.getRoleCodeByUserId(userId);

        QueryWrapper<Project> queryWrapper = QueryGenerator.initQueryWrapper(project, req.getParameterMap());
        if (!Check.isNull(projectName)) {
            queryWrapper.like("project_name", projectName);
        }
        queryWrapper.orderByDesc("create_time");
        Page<Project> page = new Page<>(pageNo, pageSize);
        IPage<Project> pageList = null;
        // 查询所有
        if (CtopRoleCodeConstant.COMMON_ROLE_CODE_ADMIN.equals(roleCode)  // 管理员
                || roleCode.equals("general_manager_of_operations")       // 运营总经理
                || roleCode.contains("sale")                              // 是所有销售相关角色
                || roleCode.equals("kuaishou_design_director")            // 快手设计总监
                || roleCode.equals("toutiao_graphic_design_supervisor")) {
            pageList = projectService.page(page, queryWrapper);
            // 经理岗查询所在项目
        } else if ("kuaishou_operations_director".equals(roleCode)  // 快手运营总监
                || "kuaishou_operations_manager".equals(roleCode)   // 快手运营经理
                || "toutiao_operations_director".equals(roleCode)  // 头条运营总监
                || "toutiao_operations_manager".equals(roleCode)  // 头条运营经理
                || "kuaishou_filming_supervisor".equals(roleCode) // 快手拍摄主管
                || "kuaishou_design_team_leader".equals(roleCode) // 快手设计组长
                || "kuaishou_design_manager".equals(roleCode)   // 快手设计经理
                || "toutiao_design_manager".equals(roleCode)) { //  头条设计经理
            List<String> userList = userService.getUserListByLeaderId(userId);
            userList.add(userId);
            List<Long> projectIds = memberService.selectProjectsByUserIds(userList);
            if (!Check.isNull(projectIds)) {
                queryWrapper.in("id", projectIds);
                pageList = projectService.page(page, queryWrapper);
            }

            // 剪辑 编导 平面 拍摄 查询自己为设计负责人项目
        } else if ("kuaishou_edit".equals(roleCode)  // 快手剪辑
                || "kuaishou_shoot".equals(roleCode) // 快手拍摄
                || "kuaishou_choreographer".equals(roleCode) // 快手编导
                || "toutiao_edit".equals(roleCode) // 头条后期
                || "toutiao_shoot".equals(roleCode) // 头条拍摄
                || "toutiao_choreographe".equals(roleCode) // 头条编导
                || "toutiao_flat".equals(roleCode)) {// 头条平面
            queryWrapper.eq("design_responsible_id", userId);
            pageList = projectService.page(page, queryWrapper);
            // 运营 运营助理查询 自己负责项目
        } else if ("kuaishou_operation".equals(roleCode) // 快手运营
                || "Kuaishou_operations_assistant".equals(roleCode) // 快手运营助理
                || "toutiao_operation".equals(roleCode)   //  头条运营
                || "toutiao_operations_assistant".equals(roleCode)) {// 头条运营助理
            queryWrapper.eq("responsible_id", userId);
            pageList = projectService.page(page, queryWrapper);
        } else {
            queryWrapper.eq("responsible_id", userId);
            pageList = projectService.page(page, queryWrapper);
        }
        List<Project> records = pageList.getRecords();
        records = transferAccountService.getTransferInfoListByList(records);
        pageList.setRecords(records);
        result.setResult(pageList);
        return result;
    }


    /**
     * 分页列表查询
     *
     * @param project
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "项目-分页列表查询")
    @ApiOperation(value = "项目-分页列表查询", notes = "项目-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<Project>> queryPageList(Project project,
                                                @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                HttpServletRequest req) {
        Result<IPage<Project>> result = new Result<>();
        String projectName = project.getProjectName();
        project.setProjectName(null);
        QueryWrapper<Project> queryWrapper = QueryGenerator.initQueryWrapper(project, req.getParameterMap());
        Page<Project> page = new Page<>(pageNo, pageSize);
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        String roleCode = sysRoleService.getRoleCodeByUserId(sysUser.getId());
        if ("operator".equals(roleCode) || "kuaishouOperationManager".equals(roleCode) || "touTiaoOperationManager".equals(roleCode)) {
            queryWrapper.eq("responsible_id", sysUser.getId());

        } else if ("plane".equals(roleCode) || "plan".equals(roleCode) || "shot".equals(roleCode) || "clip".equals(roleCode) || "designTeamLeader".equals(roleCode) || "planeLeader".equals(roleCode)) {
            queryWrapper.eq("design_responsible_id", sysUser.getId());
        }
        if (!Check.isNull(projectName)) {
            queryWrapper.like("project_name", projectName);
        }
        queryWrapper.orderByDesc("create_time");
        IPage<Project> pageList = projectService.page(page, queryWrapper);
        List<Project> records = pageList.getRecords();
        records = transferAccountService.getTransferInfoListByList(records);
        pageList.setRecords(records);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param project
     * @return
     */
    @Autowired
    private ISysUserService sysUserService;

    @AutoLog(value = "项目-添加")
    @ApiOperation(value = "项目-添加", notes = "项目-添加")
    @PostMapping(value = "/add")
    public Result<Project> add(@RequestBody Project project) {
        Result<Project> result = new Result<>();
        try {
            String advertiserId = project.getAdvertiserId();
            Advertiser advertiser = advertiserService.getById(advertiserId);
            Map<String, String> memberMap = new HashMap<>();
            if (!Check.isNull(advertiser)) {
                project.setCompanyId(advertiser.getCompanyId());
            }
          /*  SysCategory category = sysCategoryService.getById(project.getIndustryId());
            if (null != category) {
                project.setIndustryCode(category.getCode());
                project.setIndustryId(category.getId());
            }*/
            SysUser sysUser = sysUserService.getById(project.getResponsibleId());
            if (!Check.isNull(sysUser)) {
                //  project.setResponsibleName(sysUser.getRealname());
                memberMap.put(sysUser.getId(), sysUser.getRealname());
            }

            SysUser designSysUser = sysUserService.getById(project.getDesignResponsibleId());
            if (!Check.isNull(designSysUser)) {
                // project.setDesignResponsibleName(designSysUser.getRealname());
                memberMap.put(designSysUser.getId(), designSysUser.getRealname());
            }
            projectService.save(project);

            Long id = project.getId();
            for (Map.Entry<String, String> projectMember : memberMap.entrySet()) {
                String userId = projectMember.getKey();
                String userName = projectMember.getValue();
                Map<String, Object> map = new HashMap<>();
                map.put("project_id", id);
                map.put("user_id", userId);
                projectMemberService.removeByMap(map);
                ProjectMember member = new ProjectMember();
                member.setProjectId(id);
                member.setProjectName(project.getProjectName());
                member.setUserId(userId);
                member.setUserName(userName);
                String roleCode = sysRoleService.getRoleCodeByUserId(userId);
                if (!Check.isNull(roleCode)) {
                    member.setRoleCode(roleCode);
                }
                projectMemberService.save(member);
            }
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            result.error500("操作失败");
        }
        return result;
    }

    @Autowired
    private ISysRoleExtService sysRoleService;
    @Autowired
    private IMaterialInfoService materialInfoService;

    /**
     * 编辑
     *
     * @param project
     * @return
     */
    @AutoLog(value = "项目-编辑")
    @ApiOperation(value = "项目-编辑", notes = "项目-编辑")
    @PutMapping(value = "/edit")
    public Result<Project> edit(@RequestBody Project project) {
        Result<Project> result = new Result<>();
        SysUser sysUser = sysUserService.getById(project.getResponsibleId());
        if (!Check.isNull(sysUser)) {
            //     project.setResponsibleName(sysUser.getRealname());
            projectMemberService.addMember(sysUser, project);
        }

       /* SysCategory category = sysCategoryService.getById(project.getIndustryId());
        if (null != category) {
            project.setIndustryCode(category.getCode());
            project.setIndustryId(category.getId());
        }*/
        SysUser designSysUser = sysUserService.getById(project.getDesignResponsibleId());
        if (!Check.isNull(designSysUser)) {
            //      project.setDesignResponsibleName(designSysUser.getRealname());
            projectMemberService.addMember(designSysUser, project);
        }
        Project projectEntity = projectService.getById(project.getId());
        if (projectEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = projectService.updateById(project);
            QueryWrapper<ProjectMember> memberQueryWrapper = new QueryWrapper<>();
            memberQueryWrapper.eq("project_id", project.getId());
            ProjectMember updateMember = new ProjectMember();
            updateMember.setProjectName(project.getProjectName());
            projectMemberService.update(updateMember, memberQueryWrapper);

            if (!projectEntity.getSupplierCode().equals(project.getSupplierCode())) {
                QueryWrapper<MaterialInfo> materialInfoQueryWrapper = new QueryWrapper<>();
                materialInfoQueryWrapper.eq("project_id", project.getId());
                MaterialInfo updateMaterialInfo = new MaterialInfo();
                updateMaterialInfo.setSupplierCode(project.getSupplierCode());
                materialInfoService.update(updateMaterialInfo, materialInfoQueryWrapper);
            }

            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }


    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "项目-通过id删除")
    @ApiOperation(value = "项目-通过id删除", notes = "项目-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            projectService.removeById(id);
            Map<String, Object> deleteMap = new HashMap<>();
            deleteMap.put("project_id", id);
            projectMemberService.removeByMap(deleteMap);
            userAllocationService.removeByMap(deleteMap);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "项目-批量删除")
    @ApiOperation(value = "项目-批量删除", notes = "项目-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<Project> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<Project> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.projectService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "项目-通过id查询")
    @ApiOperation(value = "项目-通过id查询", notes = "项目-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<Project> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<Project> result = new Result<>();
        Project project = projectService.getById(id);
        if (project == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(project);
            result.setSuccess(true);
        }
        return result;
    }
}
